"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.express = void 0;
var code_builder_1 = require("../../../../helpers/code-builder");
exports.express = {
    info: {
        key: 'express',
        title: 'Express',
        link: 'https://expressjs.com/',
        description: 'ReadMe Metrics Webhooks SDK usage on Express'
    },
    convert: function (_a, options) {
        var secret = _a.secret, security = _a.security, server = _a.server;
        var opts = __assign({ indent: '  ' }, options);
        var _b = new code_builder_1.CodeBuilder({ indent: opts.indent }), blank = _b.blank, join = _b.join, push = _b.push, ranges = _b.ranges, variable = _b.variable;
        push("import express from 'express';");
        push("import readme from 'readmeio';");
        blank();
        push('const app = express();');
        blank();
        push('// Your ReadMe secret');
        push("const secret = '".concat(secret, "';"));
        blank();
        push("app.post('/webhook', express.json({ type: 'application/json' }), async (req, res) => {");
        push('// Verify the request is legitimate and came from ReadMe.', 1);
        push("const signature = req.headers['readme-signature'];", 1);
        blank();
        push('try {', 1);
        push('readme.verifyWebhook(req.body, signature, secret);', 2);
        push('} catch (e) {', 1);
        push('// Handle invalid requests', 2);
        push('return res.status(401).json({ error: e.message });', 2);
        push('}', 1);
        blank();
        push('// Fetch the user from the database and return their data for use with OpenAPI variables.', 1);
        push('// const user = await db.find({ email: req.body.email })', 1);
        push('return res.json({', 1);
        if (server.length) {
            push('// OAS Server variables', 2);
            server.forEach(function (data) {
                push("".concat(data.name, ": '").concat(data["default"] || data["default"] === '' ? data["default"] : data.name, "',"), 2);
                variable('server', data.name);
            });
        }
        if (server.length && security.length) {
            blank();
        }
        if (security.length) {
            push('// OAS Security variables', 2);
            security.forEach(function (data) {
                if (data.type === 'http') {
                    // Only HTTP Basic auth has any special handling for supplying auth.
                    if (data.scheme === 'basic') {
                        push("".concat(data.name, ": { user: 'user', pass: 'pass' },"), 2);
                        variable('security', data.name);
                        return;
                    }
                }
                push("".concat(data.name, ": '").concat(data["default"] || data["default"] === '' ? data["default"] : data.name, "',"), 2);
                variable('security', data.name);
            });
        }
        push('});', 1);
        push('});');
        blank();
        push('app.listen(4000);');
        return {
            ranges: ranges(),
            snippet: join()
        };
    }
};
